package logwire.web.bo.proxy;


import logwire.core.bo.handler.OperationHandler;
import logwire.core.bo.handler.TypeOperationHandler;
import logwire.web.bo.BoRelationDefinition;
import logwire.web.security.SecurityUtil;
import logwire.web.security.TenantUser;
import net.bytebuddy.implementation.bind.annotation.AllArguments;
import net.bytebuddy.implementation.bind.annotation.Origin;
import net.bytebuddy.implementation.bind.annotation.SuperMethod;
import net.bytebuddy.implementation.bind.annotation.This;

import java.lang.reflect.Method;
import java.util.Comparator;
import java.util.List;


public class TypeOperationInterceptor {

    private BoRelationDefinition boRelationDefinition;

    public TypeOperationInterceptor(BoRelationDefinition boRelationDefinition) {
        this.boRelationDefinition = boRelationDefinition;

    }

    public Object interceptor(@This Object proxy, @Origin Method method,
                              @SuperMethod Method superMethod,
                              @AllArguments Object[] args) throws Exception {
        if (superMethod != null) {
            return superMethod.invoke(proxy, args);
        }
        return doHandler(method, args);
    }

    /**
     * 根据methodName找到对应的handler
     */
    private Object doHandler(Method method, Object[] args) {
        List<OperationHandler> typeOperationHandlers = boRelationDefinition.getTypeOperationHandlers();
        OperationHandler operationHandler = typeOperationHandlers.stream()
                .filter(typeOperationHandler -> {
                    return method.getName().equals(typeOperationHandler.getOperation());
                })
                .max(Comparator.comparing(OperationHandler::getOrder)).get();

        TenantUser currentUser = SecurityUtil.currentUser();
       /* logger.debug("Execute operation {} start", new Object[]{operationName});
        String namespace = logwire.core.resource.Util.operationName(thisOperable.getOperableName(), operationName);
        actionContext.emit(namespace, BuiltinEventNames.OPERATION_BEFORE.name(), currentUser.getDomain(), new OperationEventData(thisOperable, inputArgs, null));
*/
        return ((TypeOperationHandler) operationHandler).execute(boRelationDefinition.getBoClass(), args);

    }

}

